/* $Id: read_libps.c,v 1.2 2000/01/06 22:16:11 ben Exp $ */
/*
 * Copyright (c) 2000 Ben Harris.  All rights reserved
 *
 * Redistribution and use in source and binary forms of parts of or the
 * whole original or derived work are permitted provided that the
 * original work is properly attributed to the author. The name of the
 * author may not be used to endorse or promote products derived from
 * this software without specific prior written permission. This work
 * is provided "as is" and without any express or implied warranties.
 */
/*
 * read_libps.c -- read process list from GNU Hurd libps
 * XXX untested
 */

#define _GNU_SOURCE /* Including <hurd.h> without this fails */

#include <config.h>
#include <pstree.h>

#include <sys/types.h>

#include <hurd.h>
#include <ps.h>

#ifdef HAVE_ERR_H
#include <err.h>
#endif

static error_t read_libps_entry(struct proc_stat *);

/* Get process tree from Hurd libps */
void read_libps(void)
{
    struct ps_context *pc;
    struct proc_stat_list *pp;
    struct proc_stat **proc_stats;
    unsigned int num_procs, i;
    error_t err;

    err = ps_context_create(getproc(), &pc);
    if (err != 0)
	errx(1, "ps_context_create: %s", strerror(err));
    err = proc_stat_list_create(pc, &pp);
    if (err != 0)
	errx(1, "ps_stat_list_create: %s", strerror(err));
    err = proc_stat_list_add_all(pp, &proc_stats, &num_procs);
    if (err != 0)
	errx(1, "ps_stat_list_add_all: %s", strerror(err));
    for (i = 0; i < num_procs; i++)
	read_libps_entry(proc_stats[i]);
    free(proc_stats);
    ps_context_free(pc);
}

static error_t read_libps_entry(struct proc_stat *ps)
{
    error_t err;
    ps_flags_t want_flags;

    /* We want proc_info for PPID and args for argv[0]. */
    want_flags = PSTAT_PID | PSTAT_PROC_INFO | PSTAT_OWNER_UID | PSTAT_ARGS; 
    err = proc_stat_set_flags(ps, want_flags);
    if (err != 0)
	errx(1, "proc_stat_set_flags: %s");
    if (proc_stat_flags(ps) != want_flags)
	return 0; /* Assume process has vanished */
    if (print_args)
	add_proc_args(proc_stat_args(ps), proc_stat_pid(ps),
		 proc_stat_proc_info(ps)->ppid, proc_stat_owner_uid(ps),
		 proc_stat_args(ps), proc_stat_args_len(ps));
    else
	add_proc(proc_stat_args(ps), proc_stat_pid(ps),
		 proc_stat_proc_info(ps)->ppid, proc_stat_owner_uid(ps),
		 0, NULL);
    return 0;
}

/*
 * Local variables:
 * C-file-style: "bsd"
 * End:
 */
