/* $Id: read_proc_solaris.c,v 1.1 1999/12/30 02:40:42 ben Exp $ */
/*
 * Copyright (c) Ben Harris, 1999.  All rights reserved
 *
 * Redistribution and use in source and binary forms of parts of or the
 * whole original or derived work are permitted provided that the
 * original work is properly attributed to the author. The name of the
 * author may not be used to endorse or promote products derived from
 * this software without specific prior written permission. This work
 * is provided "as is" and without any express or implied warranties.
 */
/*
 * read_proc_solaris.c - Read process data from Solaris (>=2.6) procfs
 */

#include "config.h"
#include <pstree.h>

#include <sys/stat.h>

#include <fcntl.h>
#include <procfs.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static void read_proc_entry_solaris(char const *name)
{
    struct psinfo psi;
    size_t argslen;
    char *p;
    int fd;
    char *path;

    /* Can't use chdir because of Solaris old-proc compatibility */
    path = malloc(strlen(name) + 7 + 1);
    if (path != NULL) {
	strcpy(path, name);
	strcat(path, "/psinfo");
	if ((fd = open(path, O_RDONLY)) >= 0) {
	    if (read(fd, &psi, sizeof(psi)) == sizeof(psi)) {
		argslen =  strlen(psi.pr_psargs) + 1;
		p = psi.pr_psargs;
		/* XXX: Nasty kludge */
		while (p = strchr(p, ' '))
		    *p++ = 0;
		add_proc_args(psi.pr_fname, psi.pr_pid, psi.pr_ppid,
			      psi.pr_uid, psi.pr_psargs, argslen);
	    }
	    close(fd);
	}
	free(path);
    }
}

void read_proc_solaris(void)
{
    struct stat st;

    /* Quick sanity check */
    if (stat("/proc/self/psinfo", &st) == 0)
	read_proc_generic("/proc", &read_proc_entry_solaris);
}

/*
 * Local variables:
 * C-file-style: "bsd"
 * End:
 */
